@@ -221,23 +221,20 @@ compress_stats(const Config& config,
     config.cache_dir(),
     [&](const std::string& subdir,
         const Util::ProgressReceiver& sub_progress_receiver) {
-      std::vector<std::shared_ptr<CacheFile>> files;
-      Util::get_level_1_files(
-        subdir,
-        [&](double progress) { sub_progress_receiver(progress / 2); },
-        files);
+      const std::vector<CacheFile> files = Util::get_level_1_files(
+        subdir, [&](double progress) { sub_progress_receiver(progress / 2); });
 
       for (size_t i = 0; i < files.size(); ++i) {
         const auto& cache_file = files[i];
-        on_disk_size += cache_file->lstat().size_on_disk();
+        on_disk_size += cache_file.lstat().size_on_disk();
 
         try {
-          auto file = open_file(cache_file->path(), "rb");
-          auto reader = create_reader(*cache_file, file.get());
-          compr_size += cache_file->lstat().size();
+          auto file = open_file(cache_file.path(), "rb");
+          auto reader = create_reader(cache_file, file.get());
+          compr_size += cache_file.lstat().size();
           content_size += reader->content_size();
         } catch (Error&) {
-          incompr_size += cache_file->lstat().size();
+          incompr_size += cache_file.lstat().size();
         }
 
         sub_progress_receiver(1.0 / 2 + 1.0 * i / files.size() / 2);
@@ -290,27 +287,26 @@ compress_recompress(Context& ctx,
     ctx.config.cache_dir(),
     [&](const std::string& subdir,
         const Util::ProgressReceiver& sub_progress_receiver) {
-      std::vector<std::shared_ptr<CacheFile>> files;
-      Util::get_level_1_files(
-        subdir,
-        [&](double progress) { sub_progress_receiver(0.1 * progress); },
-        files);
+      std::vector<CacheFile> files =
+        Util::get_level_1_files(subdir, [&](double progress) {
+          sub_progress_receiver(0.1 * progress);
+        });
 
       auto stats_file = subdir + "/stats";
 
       for (size_t i = 0; i < files.size(); ++i) {
         const auto& file = files[i];
 
-        if (file->type() != CacheFile::Type::unknown) {
+        if (file.type() != CacheFile::Type::unknown) {
           thread_pool.enqueue([&statistics, stats_file, file, level] {
             try {
-              recompress_file(statistics, stats_file, *file, level);
+              recompress_file(statistics, stats_file, file, level);
             } catch (Error&) {
               // Ignore for now.
             }
           });
         } else {
-          statistics.update(0, 0, 0, file->lstat().size());
+          statistics.update(0, 0, 0, file.lstat().size());
         }
 
         sub_progress_receiver(0.1 + 0.9 * i / files.size());
